/** コードに使用できる型の定義。　*/
export type EntryKey = number | string;
/** コードに対応する値に使用できる型の定義。
 *
 * コードの値に ResourceMap を指定できるようにしているのは、コードの体系が枝番のようなサブ・コードを持つ場合を想定しているため。
 */
export type EntryValue = boolean | Function | EntryKey | ResourceMap | boolean[] | Function[] | number[] | string[] | object[];
/** コードとその値の対応を定義するマップの型。
 *
 * コードは string か number をとり、コードの値は string、 number、Function,、配列、ResourceMap をとる。
 */
export type ResourceMap = {
    [entryKey: EntryKey]: EntryValue;
};
/** 指定されたオブジェクトの型が、マップのコードの型であることを判定する。
 * @param entryValue マップから取得したオブジェクトを指定する。
 * @returns 指定されたオブジェクトの型が、number か string のいずれかであれば true を返す。
 */
export declare const isResourceMapEntryKey: (entryValue: EntryValue) => boolean;
/** 指定されたオブジェクトが、コードとその値の対応を定義するマップの「葉」であることを判定する。
 * @param entryValue マップから取得したオブジェクトを指定する。
 * @returns 指定されたオブジェクトの型が、配列、boolean、Function、number、string のいずれかであれば true を返す。
 */
export declare const isResourceMapLeaf: (entryValue: EntryValue) => boolean;
/** 指定されたオブジェクトが、コードとその値の対応を定義するマップの「枝」であることを判定する。
 * @param entryValue マップから取得したオブジェクトを指定する。
 * @returns 指定されたオブジェクトが、コードとその値の対応を定義するマップの「葉」でなければ true を返す。
 */
export declare const isResourceMapTwig: (entryValue: EntryValue) => boolean;
/** コードとその値を定義したマップを新規に作成する。
 * @returns 空のマップを返す。
 */
export declare const create: () => ResourceMap;
/** コードとその値を定義したマップから、指定したコードの配列に対応する値を取得する。
 * @param resourceMap マップを指定する。
 * @param resourceKeys コード体系が枝番を持つ場合のため、コードを (string | number) の配列で指定する。
 * @returns 以下を複値 (配列) で返す。
 *
 * (1) 指定したコードに対応する値がマップに含まれていたか否かを示す boolean、
 *
 * (2) 最後にコードが対応しなかったコードの配列添え字の number 、
 * および、以下のいずれかの EntryValue 値を配列にして返す。
 *
 * (3)(a) マップに boolean、number、string、Function あるいは配列のいずれかを指定していれば、指定したマップがそのまま返される。
 *
 * (3)(b) コードをひとつも指定していなければ、指定したマップがそのまま返される。
 *
 * (3)(c) 指定したコードがマップに定義されていなければ、指定したマップがそのままか、あるいは、定義が一致した最後のコードに対応する値が返される。
 *
 * (3)(d) (3)の(a)～(c)のいずれにも該当しない場合は指定したコードに対応する値がマップに含まれていたと判定し、
 * 指定したマップで定義したコードのすべてに対応する boolean、number、string、Function
 * あるいは配列のいずれかの型の値が返される。
 */
export declare const search: (resourceMap: EntryValue, resourceKeys: EntryKey[]) => EntryValue[];
/** コードとその値を定義したマップをクリアする。
 *
 * 複数のコードで指定された値がマップである場合には、それも再帰的にクリアすることもできる。
 *
 * 複数のコードを一つも指定しない場合は、マップのすべてのエントリーをクリアする。
 * @param resourceMap マップを指定する。
 * @param recursive 再帰的にマップをクリアする場合は true を指定する。
 * @param resourceKeys コード体系が枝番を持つ場合のため、複数のコードを配列で指定できる。
 * @returns クリアしたマップを返す。
 */
export declare const clean: (resourceMap: ResourceMap, recursive: boolean, resourceKeys: EntryKey[]) => ResourceMap;
/** コードとその値を定義したマップをクリアする。
 *
 * 引数 recursive に true を指定することで、複数のコードで指定された値がマップであれば、それを再帰的にクリアできる。
 *
 * 複数のコードを一つも指定しない場合は、マップのすべてのエントリーをクリアする。
 * @param resourceMap マップを指定する。
 * @param recursive 再帰的にマップをクリアする場合は true を指定する。
 * @param resourceKeys コード体系が枝番を持つ場合のため、複数のコードを可変引数で指定できる。
 * @returns クリアしたマップを返す。
 */
export declare const cleanUp: (resourceMap: ResourceMap, recursive: boolean, ...resourceKeys: EntryKey[]) => ResourceMap;
/**
 * コードとその値を定義したマップにエントリを登録する。
 * @param resourceMap 登録する対象のマップ
 * @param entryValue 登録するエントリの値
 * @param resourceKeys 登録するエントリのキーをコードの配列で指定する。
 * @returns エントリを登録したマップ
 * @throws エントリのキーが指定されていない、エントリが既にマップに登録されている場合、あるいは、コードの並びに対応するマップが存在しない場合に例外を投げる。
 */
export declare const insert: (resourceMap: ResourceMap, entryValue: EntryValue, resourceKeys: EntryKey[], functonName?: string) => ResourceMap;
/**
 * コードとその値を定義したマップにエントリを登録する。
 * @param resourceMap 登録する対象のマップ
 * @param entryValue 登録するエントリの値
 * @param resourceKeys 登録するエントリのキーをコードの可変引数で指定する。
 * @returns エントリを登録したマップ
 * @throws エントリのキーが指定されていない、エントリが既にマップに登録されている場合、あるいは、コードの並びに対応するマップが存在しない場合に例外を投げる。
 */
export declare const insertEntry: (resourceMap: ResourceMap, entryValue: EntryValue, ...resourceKeys: EntryKey[]) => ResourceMap;
/** コードとその値を定義したマップを指定した複数のコードで探索した結果、最後に対応した値を指定された型 T で取得する。
 * @param resourceMap マップを指定する。
 * @param resourceKeys コード体系が枝番を持つ場合のため、コードを (string | number) の配列で指定する。
 * @returns コードに対応した値を指定された型で返す。
 * @throws 最後にコードが対応しなかったコードの配列添え字、および、指定したマップがそのままか、あるいは、定義が一致した最後のコードに対応する値を配列にして投げる。
 */
export declare const select: <T>(resourceMap: EntryValue, resourceKeys: EntryKey[]) => T;
/** コードとその値を定義したマップを指定した複数のコードで探索した結果、最後に対応した値を指定された型 T で取得する。
 * @param resourceMap マップを指定する。
 * @param resourceKeys コード体系が枝番を持つ場合のため、コードを (string | number) の可変個引数で指定する。
 * @returns コードに対応した値を指定された型で返す。
 * @throws 最後にコードが対応しなかったコードの配列添え字、および、指定したマップがそのままか、あるいは、定義が一致した最後のコードに対応する値を配列にして投げる。
 */
export declare const selectAs: <T>(resourceMap: EntryValue, ...resourceKeys: EntryKey[]) => T;
/**
 * コードとその値を定義したマップの指定したエントリの値を更新する。
 * @param resourceMap エントリの値を更新する対象のマップ
 * @param entryValue エントリを更新する値
 * @param resourceKeys 更新するエントリに対応するキーをコードの配列で指定する。
 * @returns エントリの値を更新したマップ
 * @throws 更新するエントリに対応するキーが指定されていない、あるいは、指定されたエントリで更新できるマップが存在しない場合に例外を投げる。
 */
export declare const update: (resourceMap: ResourceMap, entryValue: EntryValue, resourceKeys: EntryKey[]) => ResourceMap;
/**
 * コードとその値を定義したマップの指定したエントリの値を更新する。
 * @param resourceMap エントリの値を更新する対象のマップ
 * @param entryValue エントリを更新する値
 * @param resourceKeys 更新するエントリに対応するキーをコードの可変引数で指定する。
 * @returns エントリの値を更新したマップ
 * @throws 更新するエントリに対応するキーが指定されていない、あるいは、指定されたエントリで更新できるマップが存在しない場合に例外を投げる。
 */
export declare const updateEntry: (resourceMap: ResourceMap, entryValue: EntryValue, ...resourceKeys: EntryKey[]) => ResourceMap;
/**
 * コードとその値を定義したマップからエントリを削除する。
 * @param resourceMap エントリを削除する対象のマップ
 * @param resourceKeys 削除するエントリに対応するキーをコードの配列で指定する。
 * @returns エントリを削除したマップ
 * @throws 削除するエントリに対応するキーが指定されていない場合に例外を投げる。
 */
export declare const deleteIt: (resourceMap: ResourceMap, resourceKeys: EntryKey[]) => ResourceMap;
/**
 * コードとその値を定義したマップからエントリを削除する。
 * @param resourceMap エントリを削除する対象のマップ
 * @param resourceKeys 削除するエントリに対応するキーをコードの可変引数で指定する。
 * @returns エントリを削除したマップ
 * @throws 削除するエントリに対応するキーが指定されていない場合に例外を投げる。
 */
export declare const deleteEntry: (resourceMap: ResourceMap, ...resourceKeys: EntryKey[]) => ResourceMap;
/**
 * コードとその値を定義したマップの指定したエントリの値を更新 (エントリがなければ登録) する。
 * @param resourceMap エントリの値を更新/登録する対象のマップ
 * @param entryValue エントリを更新/登録する値
 * @param resourceKeys 更新/登録するエントリに対応するキーをコードの配列で指定する。
 * @param functionName 呼び出した関数の名称。デフォルトは 'merge'
 * @param forcedUpdate マップにすでにエントリが存在していた場合も更新するには true。デフォルト値は true
 * @param makeLink マップにすでにエントリが存在していない場合に新規登録するには true。デフォルト値は true
 * @returns エントリの値を更新/登録したマップ
 * @throws 更新/登録するエントリに対応するキーを指定していない、 forcedUpdate と makeLink の両方に false を指定している、あるいは、指定されたエントリで更新/登録できるマップが存在しない場合に例外を投げる。
 */
export declare const merge: (resourceMap: ResourceMap, entryValue: EntryValue, resourceKeys: EntryKey[], functionName?: string, forcedUpdate?: boolean, makeLink?: boolean) => ResourceMap;
/**
 * コードとその値を定義したマップの指定したエントリの値を更新 (エントリがなければ登録) する。
 * @param resourceMap エントリの値を更新/登録する対象のマップ
 * @param entryValue エントリを更新/登録する値
 * @param resourceKeys 更新/登録するエントリに対応するキーをコードの可変引数で指定する。
 * @returns エントリの値を更新/登録したマップ
 * @throws 更新/登録するエントリに対応するキーを指定していない、あるいは、指定されたエントリで更新/登録できるマップが存在しない場合に例外を投げる。
 */
export declare const mergeEntry: (resourceMap: ResourceMap, entryValue: EntryValue, ...resourceKeys: EntryKey[]) => ResourceMap;
/**
 * 文字列リテラルの並びを記述した文字列をキーのコードの配列に変換する。
 *
 * 例えば、文字列リテラル「'"3", "abc", "かさたは"'」は、string[]  の「["3", "abc", "かさたは"]」に変換される。
 * @param resourceKeyPath 文字列リテラルの並びを記述した文字列
 * @returns 変換した結果のキーのコードの配列
 * @throws 文字列リテラルの記述が不正な場合 (例えば、ダブルクォートでコードを囲っていない。カンマで区切っていない等) に例外を投げる。
 */
export declare const toEntryKeys: (resourceKeyPath: string) => EntryKey[];
/**
 * キーと string あるいは number の値のエントリだけを登録してあるマップから、値でキーを逆引きできるマップを生成する。
 * @param resourceMap 逆引きするマップを生成する元のマップ
 * @returns 逆引きするマップ
 * @throws 元のマップのいずれかのエントリの値が string でも number でもない場合、あるいは、値が重複していた場合に例外を投げる。
 */
export declare const reverse: (resourceMap: ResourceMap) => ResourceMap;
/**
 * 二つのマップが互いに一対一対応していることを確認する。
 *
 * マップはどちらともキーと値が string か number であること。
 * @param mapA 確認するマップ
 * @param mapB もうひとつの確認するマップ
 * @returns 二つのマップが互いに一対一対応している場合に true
 */
export declare const oneToOne: (mapA: ResourceMap, mapB: ResourceMap) => boolean;
/**
 * マップの deep コピーを生成する。
 * @param existValue コピー元のマップ
 * @param refillFunctions コピー元の Function 型あるいは Function[] 型の値を生成したマップに登録する場合は true を指定する。デフォルト値は false
 */
export declare const duplicateEntryValue: (existValue: EntryValue, refillFunctions?: boolean) => EntryValue;
/**
 * 二つのマップの「枝」が、それらに登録されているすべてのエントリも含めて、等しいことを確認する。
 * @param twigA マップの「枝」
 * @param twigB マップのもうひとつの「枝」
 * @returns 二つのマップの「枝」が、それらに登録されているすべてのエントリも含めて、等しい場合に true
 */
export declare const equalsEntryValue: (twigA: EntryValue, twigB: EntryValue) => boolean;
/**
 * マップが空っぽであることを確認する。
 * @param aMap 確認する対象のマップ
 * @returns マップにエントリがひとつも登録されていない場合に true
 */
export declare const isEmpty: (aMap: ResourceMap) => boolean;
/**
 * あるエントリの値のキーを変更する。
 * @param resourceMap 操作の対象とするマップ
 * @param existResourceKeys 既存のあるエントリを指定するキーをコードの配列で指定する。
 * @param resourceKeys 変更する別のエントリのキーをコードの配列で指定する。
 * @param functionName 呼び出した関数の名称。デフォルト値は 'move'
 * @returns あるエントリの値のキーを変更した結果のマップ
 * @throws 以下の場合に例外を投げる。
 *
 * (1) エントリを指定するキーが指定されていない場合
 *
 * (2) 操作元になるエントリがマップに登録されていない場合
 *
 * (3) 「枝」を実際に操作するための他の関数 (update(), insert(), deleteIt()) で例外が発生した場合
 */
export declare const move: (resourceMap: ResourceMap, existResourceKeys: EntryKey[], resourceKeys: EntryKey[], functionName?: string) => ResourceMap;
/**
 * あるエントリの値のキーを変更する。
 * @param resourceMap 操作の対象とするマップ
 * @param existResourceKeys 既存のあるエントリを指定するキーをコードの配列で指定する。
 * @param resourceKeys 変更する別のエントリのキーをコードの可変引数で指定する。
 * @returns あるエントリの値のキーを変更した結果のマップ
 * @throws 以下の場合に例外を投げる。
 *
 * (1) エントリを指定するキーが指定されていない場合
 *
 * (2) 操作元になるエントリがマップに登録されていない場合
 *
 * (3) 「枝」を実際に操作するための他の関数 (update(), insert(), deleteIt()) で例外が発生した場合
 */
export declare const moveEntry: (resourceMap: ResourceMap, existResourceKeys: EntryKey[], ...resourceKeys: EntryKey[]) => ResourceMap;
/**
 * あるエントリの値の shallow コピーで別のあらたなエントリを作成する。
 *
 * @param resourceMap 操作の対象とするマップ
 * @param existResourceKeys 既存のあるエントリを指定するキーをコードの配列で指定する。
 * @param resourceKeys shallow コピーを作成する別のエントリのキーをコードの配列で指定する。
 * @param functionName 呼び出した関数の名称。デフォルト値は 'link'
 * @returns shallow コピーであらたなエントリを作成した結果のマップ
 * @throws 以下の場合に例外を投げる。
 *
 * (1) エントリを指定するキーが指定されていない場合
 *
 * (2) 操作元になるエントリがマップに登録されていない場合
 *
 * (3) あらたに作成するエントリがすでにマップに登録されている場合
 *
 * (4) 「枝」を実際に操作するための他の関数 (insert()) で例外が発生した場合
 */
export declare const link: (resourceMap: ResourceMap, existResourceKeys: EntryKey[], resourceKeys: EntryKey[], functionName?: string) => ResourceMap;
/**
 * あるエントリの値の shallow コピーで別のあらたなエントリを作成する。
 *
 * @param resourceMap 操作の対象とするマップ
 * @param existResourceKeys 既存のあるエントリを指定するキーをコードの配列で指定する。
 * @param resourceKeys shallow コピーを作成する別のエントリのキーをコードの可変引数で指定する。
 * @returns shallow コピーであらたなエントリを作成した結果のマップ
 * @throws 以下の場合に例外を投げる。
 *
 * (1) エントリを指定するキーが指定されていない場合
 *
 * (2) 操作元になるエントリがマップに登録されていない場合
 *
 * (3) あらたに作成するエントリがすでにマップに登録されている場合
 *
 * (4) 「枝」を実際に操作するための他の関数 (insert()) で例外が発生した場合
 */
export declare const linkEntry: (resourceMap: ResourceMap, existResourceKeys: EntryKey[], ...resourceKeys: EntryKey[]) => ResourceMap;
/**
 * あるエントリの値の deep コピーで別のあらたなエントリを作成する。
 * @param resourceMap 操作の対象とするマップ
 * @param existResourceKeys 既存のあるエントリを指定するキーをコードの配列で指定する。
 * @param resourceKeys 更新/作成/変更する別のエントリのキーをコードの配列で指定する。
 * @param refillFunctions あるエントリの値のコピーにコピー元に登録されている関数もコピーする場合に true を指定する。デフォルト値は false
 * @param functionName 呼び出した関数の名称。デフォルト値は 'copy'
 * @returns deep コピーであらたなエントリを作成した結果のマップ
 * @throws 以下の場合に例外を投げる。
 *
 * (1) エントリを指定するキーが指定されていない場合
 *
 * (2) 操作元になるエントリがマップに登録されていない場合
 *
 * (3) あらたに作成するエントリがすでにマップに登録されている場合
 *
 * (4) 「枝」を実際に操作するための他の関数 (duplicateEntryValue(), insert()) で例外が発生した場合
 */
export declare const copy: (resourceMap: ResourceMap, existResourceKeys: EntryKey[], resourceKeys: EntryKey[], refillFunctions?: boolean, functionName?: string) => ResourceMap;
/**
 * あるエントリの値の deep コピーで別のあらたなエントリを作成する。
 * @param resourceMap 操作の対象とするマップ
 * @param refillFunctions あるエントリの値のコピーにコピー元に登録されている関数もコピーする場合に true を指定する。
 * @param existResourceKeys 既存のあるエントリを指定するキーをコードの配列で指定する。
 * @param resourceKeys 更新/作成/変更する別のエントリのキーをコードの可変引数で指定する。
 * @returns deep コピーであらたなエントリを作成した結果のマップ
 * @throws 以下の場合に例外を投げる。
 *
 * (1) エントリを指定するキーが指定されていない場合
 *
 * (2) 操作元になるエントリがマップに登録されていない場合
 *
 * (3) あらたに作成するエントリがすでにマップに登録されている場合
 *
 * (4) 「枝」を実際に操作するための他の関数 (duplicateEntryValue(), insert()) で例外が発生した場合
 */
export declare const copyEntry: (resourceMap: ResourceMap, refillFunctions: boolean, existResourceKeys: EntryKey[], ...resourceKeys: EntryKey[]) => ResourceMap;
