"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.copyEntry = exports.copy = exports.linkEntry = exports.link = exports.moveEntry = exports.move = exports.isEmpty = exports.equalsEntryValue = exports.duplicateEntryValue = exports.oneToOne = exports.reverse = exports.toEntryKeys = exports.mergeEntry = exports.merge = exports.deleteEntry = exports.deleteIt = exports.updateEntry = exports.update = exports.selectAs = exports.select = exports.insertEntry = exports.insert = exports.cleanUp = exports.clean = exports.search = exports.create = exports.isResourceMapTwig = exports.isResourceMapLeaf = exports.isResourceMapEntryKey = void 0;
/** 指定されたオブジェクトの型が、マップのコードの型であることを判定する。
 * @param entryValue マップから取得したオブジェクトを指定する。
 * @returns 指定されたオブジェクトの型が、number か string のいずれかであれば true を返す。
 */
const isResourceMapEntryKey = (entryValue) => {
    return ['number', 'string'].indexOf(typeof entryValue) >= 0;
};
exports.isResourceMapEntryKey = isResourceMapEntryKey;
/** 指定されたオブジェクトが、コードとその値の対応を定義するマップの「葉」であることを判定する。
 * @param entryValue マップから取得したオブジェクトを指定する。
 * @returns 指定されたオブジェクトの型が、配列、boolean、Function、number、string のいずれかであれば true を返す。
 */
const isResourceMapLeaf = (entryValue) => {
    return Array.isArray(entryValue)
        || (0, exports.isResourceMapEntryKey)(entryValue)
        || ['boolean', 'function'].indexOf(typeof entryValue) >= 0;
};
exports.isResourceMapLeaf = isResourceMapLeaf;
/** 指定されたオブジェクトが、コードとその値の対応を定義するマップの「枝」であることを判定する。
 * @param entryValue マップから取得したオブジェクトを指定する。
 * @returns 指定されたオブジェクトが、コードとその値の対応を定義するマップの「葉」でなければ true を返す。
 */
const isResourceMapTwig = (entryValue) => {
    return !(0, exports.isResourceMapLeaf)(entryValue);
};
exports.isResourceMapTwig = isResourceMapTwig;
/** コードとその値を定義したマップを新規に作成する。
 * @returns 空のマップを返す。
 */
const create = () => {
    return {};
};
exports.create = create;
/** コードとその値を定義したマップから、指定したコードの配列に対応する値を取得する。
 * @param resourceMap マップを指定する。
 * @param resourceKeys コード体系が枝番を持つ場合のため、コードを (string | number) の配列で指定する。
 * @returns 以下を複値 (配列) で返す。
 *
 * (1) 指定したコードに対応する値がマップに含まれていたか否かを示す boolean、
 *
 * (2) 最後にコードが対応しなかったコードの配列添え字の number 、
 * および、以下のいずれかの EntryValue 値を配列にして返す。
 *
 * (3)(a) マップに boolean、number、string、Function あるいは配列のいずれかを指定していれば、指定したマップがそのまま返される。
 *
 * (3)(b) コードをひとつも指定していなければ、指定したマップがそのまま返される。
 *
 * (3)(c) 指定したコードがマップに定義されていなければ、指定したマップがそのままか、あるいは、定義が一致した最後のコードに対応する値が返される。
 *
 * (3)(d) (3)の(a)～(c)のいずれにも該当しない場合は指定したコードに対応する値がマップに含まれていたと判定し、
 * 指定したマップで定義したコードのすべてに対応する boolean、number、string、Function
 * あるいは配列のいずれかの型の値が返される。
 */
const search = (resourceMap, resourceKeys) => {
    return resourceKeys.length > 0
        ? resourceKeys.reduce(([foundIt, lastIndex, map], key) => ((foundIt
            && (0, exports.isResourceMapTwig)(map)
            && (foundIt = key in map)) ? [foundIt, 1 + lastIndex, map[key]]
            : [false, lastIndex, map]), [true, 0, resourceMap])
        : [false, -1, resourceMap];
};
exports.search = search;
/** コードとその値を定義したマップをクリアする。
 *
 * 複数のコードで指定された値がマップである場合には、それも再帰的にクリアすることもできる。
 *
 * 複数のコードを一つも指定しない場合は、マップのすべてのエントリーをクリアする。
 * @param resourceMap マップを指定する。
 * @param recursive 再帰的にマップをクリアする場合は true を指定する。
 * @param resourceKeys コード体系が枝番を持つ場合のため、複数のコードを配列で指定できる。
 * @returns クリアしたマップを返す。
 */
const clean = (resourceMap, recursive, resourceKeys) => {
    let [foundIt, lastIndex, result] = (0, exports.search)(resourceMap, resourceKeys);
    if (resourceKeys.length == 0 || foundIt) {
        Object.keys(result).forEach((key) => {
            let val = result[key];
            if (recursive && (0, exports.isResourceMapTwig)(val)) {
                (0, exports.clean)(val, recursive, []);
            }
            delete result[key];
        });
    }
    return resourceMap;
};
exports.clean = clean;
/** コードとその値を定義したマップをクリアする。
 *
 * 引数 recursive に true を指定することで、複数のコードで指定された値がマップであれば、それを再帰的にクリアできる。
 *
 * 複数のコードを一つも指定しない場合は、マップのすべてのエントリーをクリアする。
 * @param resourceMap マップを指定する。
 * @param recursive 再帰的にマップをクリアする場合は true を指定する。
 * @param resourceKeys コード体系が枝番を持つ場合のため、複数のコードを可変引数で指定できる。
 * @returns クリアしたマップを返す。
 */
const cleanUp = (resourceMap, recursive, ...resourceKeys) => {
    return (0, exports.clean)(resourceMap, recursive, resourceKeys);
};
exports.cleanUp = cleanUp;
/** 例外で投げる配列の開始文字列を生成する
 * @param funcName 例外を投げる関数の名称
 * @returns 例外で投げる配列の開始文字列
 */
const exceptionMsg = (funcName) => {
    return 'Exception:' + funcName + '(';
};
/** 例外で投げる配列の終了を示す文字列 */
const EXCEPTION_MSG_END = ')';
/** 例外で投げる配列のローカル変数並びの開始を示す文字列 */
const EXCEPTION_MSG_VARIABLES = EXCEPTION_MSG_END + ' variables (';
/** 例外で投げる配列の原因例外情報の開始を示す文字列 */
const causedExceptionMsg = (e) => {
    return EXCEPTION_MSG_END + ' (Caused Exception (' + e + EXCEPTION_MSG_END;
};
// 【CRUD & Merge...】
/**
 * コードとその値を定義したマップにエントリを登録する。
 * @param resourceMap 登録する対象のマップ
 * @param entryValue 登録するエントリの値
 * @param resourceKeys 登録するエントリのキーをコードの配列で指定する。
 * @param functionName 呼び出した関数の名称。デフォルト値は 'insert'
 * @returns エントリを登録したマップ
 * @throws エントリのキーが指定されていない、エントリが既にマップに登録されている場合、あるいは、コードの並びに対応するマップが存在しない場合に例外を投げる。
 */
const ins = (resourceMap, entryValue, resourceKeys, functionName = 'insert') => {
    let [map, limit] = [resourceMap, resourceKeys.length - 1];
    if (limit < 0 || (limit == 0 && resourceKeys[limit] in map)) {
        throw [exceptionMsg(functionName), resourceMap, entryValue, resourceKeys,
            EXCEPTION_MSG_VARIABLES, map, limit,
            EXCEPTION_MSG_END
        ];
    }
    if (limit > 0) {
        let [foundIt, lastIndex, result] = (0, exports.search)(map, resourceKeys.slice(0, limit));
        if ((foundIt && (0, exports.isResourceMapLeaf)(result))
            || resourceKeys[lastIndex] in result) {
            throw [exceptionMsg(functionName), resourceMap, entryValue, resourceKeys,
                EXCEPTION_MSG_VARIABLES, map, limit, foundIt, lastIndex, result,
                EXCEPTION_MSG_END
            ];
        }
        !foundIt &&
            resourceKeys.slice(lastIndex, limit).forEach((key) => {
                result[key] = (0, exports.create)();
                result = result[key];
            });
        map = result;
    }
    map[resourceKeys[limit]] = entryValue;
    return resourceMap;
};
/**
 * コードとその値を定義したマップにエントリを登録する。
 * @param resourceMap 登録する対象のマップ
 * @param entryValue 登録するエントリの値
 * @param resourceKeys 登録するエントリのキーをコードの配列で指定する。
 * @returns エントリを登録したマップ
 * @throws エントリのキーが指定されていない、エントリが既にマップに登録されている場合、あるいは、コードの並びに対応するマップが存在しない場合に例外を投げる。
 */
const insert = (resourceMap, entryValue, resourceKeys, functonName = 'insert') => {
    return ins(resourceMap, entryValue, resourceKeys);
};
exports.insert = insert;
/**
 * コードとその値を定義したマップにエントリを登録する。
 * @param resourceMap 登録する対象のマップ
 * @param entryValue 登録するエントリの値
 * @param resourceKeys 登録するエントリのキーをコードの可変引数で指定する。
 * @returns エントリを登録したマップ
 * @throws エントリのキーが指定されていない、エントリが既にマップに登録されている場合、あるいは、コードの並びに対応するマップが存在しない場合に例外を投げる。
 */
const insertEntry = (resourceMap, entryValue, ...resourceKeys) => {
    return ins(resourceMap, entryValue, resourceKeys, 'insertEntry');
};
exports.insertEntry = insertEntry;
/** コードとその値を定義したマップを指定した複数のコードで探索した結果、最後に対応した値を指定された型 T で取得する。
 * @param resourceMap マップを指定する。
 * @param resourceKeys コード体系が枝番を持つ場合のため、コードを (string | number) の配列で指定する。
 * @param functionName 呼び出した関数の名称。デフォルト値は 'select<T>'
 * @returns コードに対応した値を指定された型で返す。
 * @throws 最後にコードが対応しなかったコードの配列添え字、および、指定したマップがそのままか、あるいは、定義が一致した最後のコードに対応する値を配列にして投げる。
 */
const sel = function (resourceMap, resourceKeys, functionName = 'select<T>') {
    let [foundIt, lastIndex, result] = (0, exports.search)(resourceMap, resourceKeys);
    if (!foundIt) {
        throw [exceptionMsg(functionName), resourceMap, resourceKeys,
            EXCEPTION_MSG_VARIABLES, foundIt, lastIndex, result,
            EXCEPTION_MSG_END
        ];
    }
    return result;
};
/** コードとその値を定義したマップを指定した複数のコードで探索した結果、最後に対応した値を指定された型 T で取得する。
 * @param resourceMap マップを指定する。
 * @param resourceKeys コード体系が枝番を持つ場合のため、コードを (string | number) の配列で指定する。
 * @returns コードに対応した値を指定された型で返す。
 * @throws 最後にコードが対応しなかったコードの配列添え字、および、指定したマップがそのままか、あるいは、定義が一致した最後のコードに対応する値を配列にして投げる。
 */
const select = function (resourceMap, resourceKeys) {
    return sel(resourceMap, resourceKeys);
};
exports.select = select;
/** コードとその値を定義したマップを指定した複数のコードで探索した結果、最後に対応した値を指定された型 T で取得する。
 * @param resourceMap マップを指定する。
 * @param resourceKeys コード体系が枝番を持つ場合のため、コードを (string | number) の可変個引数で指定する。
 * @returns コードに対応した値を指定された型で返す。
 * @throws 最後にコードが対応しなかったコードの配列添え字、および、指定したマップがそのままか、あるいは、定義が一致した最後のコードに対応する値を配列にして投げる。
 */
const selectAs = function (resourceMap, ...resourceKeys) {
    return sel(resourceMap, resourceKeys, 'selectAs<T>');
};
exports.selectAs = selectAs;
/**
 * コードとその値を定義したマップから指定したエントリを削除するか、または、その値を更新する対象のエントリを探す。
 * @param functionName 呼び出した関数の名称。
 * @param resourceMap エントリの値を更新する対象のマップ
 * @param entryValue エントリを更新する値
 * @param resourceKeys 更新するエントリに対応するキーをコードの配列で指定する。
 * @returns 対象となるエントリを探した結果を以下のデータ項目の配列で返す。
 *
 * (1) 対象となるエントリが見つかったか否かを示す boolean
 *
 * (2) 対象となるエントリの EntryValue
 *
 * (3) 対象となるエントリが見つからなかった最後のキーのコード配列添え字を示す number。見つかった場合はコード配列の長さに等しくなる。
 * @throws キーを指定されなかった場合に例外を返す。
 */
const searchToModify = (functionName, resourceMap, entryValue, resourceKeys) => {
    let [canDo, map, limit] = [true, resourceMap, resourceKeys.length - 1];
    if (limit < 0) {
        throw [exceptionMsg(functionName), resourceMap, entryValue, resourceKeys,
            EXCEPTION_MSG_VARIABLES, canDo, map, limit,
            EXCEPTION_MSG_END
        ];
    }
    if (limit > 0) {
        let [foundIt, lastIndex, result] = (0, exports.search)(resourceMap, resourceKeys.slice(0, limit));
        canDo = foundIt && (0, exports.isResourceMapTwig)(result);
        map = result;
        limit = lastIndex;
    }
    return [canDo, map, limit];
};
/**
 * コードとその値を定義したマップの指定したエントリの値を更新する。
 * @param resourceMap エントリの値を更新する対象のマップ
 * @param entryValue エントリを更新する値
 * @param resourceKeys 更新するエントリに対応するキーをコードの配列で指定する。
 * @param functionName 呼び出した関数の名称。デフォルトは 'update'
 * @returns エントリの値を更新したマップ
 * @throws 更新するエントリに対応するキーが指定されていない、あるいは、指定されたエントリで更新できるマップが存在しない場合に例外を投げる。
 */
const upd = (resourceMap, entryValue, resourceKeys, functionName = 'update') => {
    let [canUpdate, map, limit] = searchToModify(functionName, resourceMap, entryValue, resourceKeys);
    if (!canUpdate || !(resourceKeys[limit] in map)) {
        throw [exceptionMsg(functionName), resourceMap, entryValue, resourceKeys,
            EXCEPTION_MSG_VARIABLES, canUpdate, map, limit,
            EXCEPTION_MSG_END
        ];
    }
    map[resourceKeys[limit]] = entryValue;
    return resourceMap;
};
/**
 * コードとその値を定義したマップの指定したエントリの値を更新する。
 * @param resourceMap エントリの値を更新する対象のマップ
 * @param entryValue エントリを更新する値
 * @param resourceKeys 更新するエントリに対応するキーをコードの配列で指定する。
 * @returns エントリの値を更新したマップ
 * @throws 更新するエントリに対応するキーが指定されていない、あるいは、指定されたエントリで更新できるマップが存在しない場合に例外を投げる。
 */
const update = (resourceMap, entryValue, resourceKeys) => {
    return upd(resourceMap, entryValue, resourceKeys);
};
exports.update = update;
/**
 * コードとその値を定義したマップの指定したエントリの値を更新する。
 * @param resourceMap エントリの値を更新する対象のマップ
 * @param entryValue エントリを更新する値
 * @param resourceKeys 更新するエントリに対応するキーをコードの可変引数で指定する。
 * @returns エントリの値を更新したマップ
 * @throws 更新するエントリに対応するキーが指定されていない、あるいは、指定されたエントリで更新できるマップが存在しない場合に例外を投げる。
 */
const updateEntry = (resourceMap, entryValue, ...resourceKeys) => {
    return upd(resourceMap, entryValue, resourceKeys, 'updateEntry');
};
exports.updateEntry = updateEntry;
/**
 * コードとその値を定義したマップからエントリを削除する。
 * @param resourceMap エントリを削除する対象のマップ
 * @param resourceKeys 削除するエントリに対応するキーをコードの配列で指定する。
 * @param functionName 呼び出した関数の名称。デフォルトは 'deleteIt'
 * @returns エントリを削除したマップ
 * @throws 削除するエントリに対応するキーが指定されていない場合に例外を投げる。
 */
const del = (resourceMap, resourceKeys, functionName = 'deleteIt') => {
    let [canDelete, map, limit] = searchToModify(functionName, resourceMap, null, resourceKeys);
    if (canDelete && resourceKeys[limit] in map) {
        delete map[resourceKeys[limit]];
    }
    return resourceMap;
};
/**
 * コードとその値を定義したマップからエントリを削除する。
 * @param resourceMap エントリを削除する対象のマップ
 * @param resourceKeys 削除するエントリに対応するキーをコードの配列で指定する。
 * @returns エントリを削除したマップ
 * @throws 削除するエントリに対応するキーが指定されていない場合に例外を投げる。
 */
const deleteIt = (resourceMap, resourceKeys) => {
    return del(resourceMap, resourceKeys);
};
exports.deleteIt = deleteIt;
/**
 * コードとその値を定義したマップからエントリを削除する。
 * @param resourceMap エントリを削除する対象のマップ
 * @param resourceKeys 削除するエントリに対応するキーをコードの可変引数で指定する。
 * @returns エントリを削除したマップ
 * @throws 削除するエントリに対応するキーが指定されていない場合に例外を投げる。
 */
const deleteEntry = (resourceMap, ...resourceKeys) => {
    return del(resourceMap, resourceKeys, 'deleteEntry');
};
exports.deleteEntry = deleteEntry;
/**
 * コードとその値を定義したマップの指定したエントリの値を更新 (エントリがなければ登録) する。
 * @param resourceMap エントリの値を更新/登録する対象のマップ
 * @param entryValue エントリを更新/登録する値
 * @param resourceKeys 更新/登録するエントリに対応するキーをコードの配列で指定する。
 * @param functionName 呼び出した関数の名称。デフォルトは 'merge'
 * @param forcedUpdate マップにすでにエントリが存在していた場合も更新するには true。デフォルト値は true
 * @param makeLink マップにすでにエントリが存在していない場合に新規登録するには true。デフォルト値は true
 * @returns エントリの値を更新/登録したマップ
 * @throws 更新/登録するエントリに対応するキーを指定していない、 forcedUpdate と makeLink の両方に false を指定している、あるいは、指定されたエントリで更新/登録できるマップが存在しない場合に例外を投げる。
 */
const merge = (resourceMap, entryValue, resourceKeys, functionName = 'merge', forcedUpdate = true, makeLink = true) => {
    let [isFound, map, limit] = searchToModify(functionName, resourceMap, entryValue, resourceKeys);
    if ((!forcedUpdate && !makeLink) || !(0, exports.isResourceMapTwig)(map)) {
        throw [exceptionMsg(functionName), resourceMap, entryValue, resourceKeys, functionName, forcedUpdate, makeLink,
            EXCEPTION_MSG_VARIABLES, isFound, map, typeof map, limit,
            EXCEPTION_MSG_END
        ];
    }
    if (isFound) {
        forcedUpdate && (map[resourceKeys[limit]] = entryValue);
    }
    else {
        makeLink && (0, exports.insert)(map, entryValue, resourceKeys.slice(limit));
    }
    return resourceMap;
};
exports.merge = merge;
/**
 * コードとその値を定義したマップの指定したエントリの値を更新 (エントリがなければ登録) する。
 * @param resourceMap エントリの値を更新/登録する対象のマップ
 * @param entryValue エントリを更新/登録する値
 * @param resourceKeys 更新/登録するエントリに対応するキーをコードの可変引数で指定する。
 * @returns エントリの値を更新/登録したマップ
 * @throws 更新/登録するエントリに対応するキーを指定していない、あるいは、指定されたエントリで更新/登録できるマップが存在しない場合に例外を投げる。
 */
const mergeEntry = (resourceMap, entryValue, ...resourceKeys) => {
    return (0, exports.merge)(resourceMap, entryValue, resourceKeys, 'mergeEntry');
};
exports.mergeEntry = mergeEntry;
// 【...CRUD & Merge】
// 【Utility...】
/**
 * 文字列リテラルの並びを記述した文字列をキーのコードの配列に変換する。
 *
 * 例えば、文字列リテラル「'"3", "abc", "かさたは"'」は、string[]  の「["3", "abc", "かさたは"]」に変換される。
 * @param resourceKeyPath 文字列リテラルの並びを記述した文字列
 * @returns 変換した結果のキーのコードの配列
 * @throws 文字列リテラルの記述が不正な場合 (例えば、ダブルクォートでコードを囲っていない。カンマで区切っていない等) に例外を投げる。
 */
const toEntryKeys = (resourceKeyPath) => {
    let s = resourceKeyPath.trim();
    if (s.indexOf('[') != 0) {
        s = '[' + s;
    }
    if (s.lastIndexOf(']') != s.length - 1) {
        s = s + ']';
    }
    try {
        return JSON.parse(s);
    }
    catch (e) {
        throw [exceptionMsg('toEntryKeys'), resourceKeyPath,
            EXCEPTION_MSG_VARIABLES, s,
            causedExceptionMsg(e)
        ];
    }
};
exports.toEntryKeys = toEntryKeys;
/**
 * キーと string あるいは number の値のエントリだけを登録してあるマップから、値でキーを逆引きできるマップを生成する。
 * @param resourceMap 逆引きするマップを生成する元のマップ
 * @returns 逆引きするマップ
 * @throws 元のマップのいずれかのエントリの値が string でも number でもない場合、あるいは、値が重複していた場合に例外を投げる。
 */
const reverse = (resourceMap) => {
    let [expMsg, rev] = [exceptionMsg('reverse'), (0, exports.create)()];
    Object.keys(resourceMap).forEach((key, index) => {
        let val = resourceMap[key];
        if (!(0, exports.isResourceMapEntryKey)(val)) {
            throw [expMsg, resourceMap,
                EXCEPTION_MSG_VARIABLES, rev, key, index, val,
                EXCEPTION_MSG_END
            ];
        }
        try {
            (0, exports.insertEntry)(rev, key, val);
        }
        catch (e) {
            throw [expMsg, resourceMap,
                EXCEPTION_MSG_VARIABLES, rev, key, index, val,
                causedExceptionMsg(e)
            ];
        }
    });
    return rev;
};
exports.reverse = reverse;
/**
 * 二つのマップが互いに一対一対応していることを確認する。
 *
 * マップはどちらともキーと値が string か number であること。
 * @param mapA 確認するマップ
 * @param mapB もうひとつの確認するマップ
 * @returns 二つのマップが互いに一対一対応している場合に true
 */
const oneToOne = (mapA, mapB) => {
    let keys = Object.keys(mapA);
    return ((keys.length == Object.keys(mapB).length)
        && keys.findIndex((key) => {
            let [valA, valB] = [mapA[key], mapB[mapA[key]]];
            return !(0, exports.isResourceMapEntryKey)(valA)
                || !(0, exports.isResourceMapEntryKey)(valB)
                || key != valB;
        }) < 0);
};
exports.oneToOne = oneToOne;
/**
 * コピー元のマップ（と、その値になっているサブ・マップ) で Function 型あるいは Function[] 型の値があれば、それをコピー先のマップに登録する。
 * @param entryValue コピー先のマップ
 * @param existValue コピー元のマップ
 * @returns コピー元の Function 型あるいは Function[] 型の値を登録したコピー先のマップ
 */
const refillFunctionValues = (entryValue, existValue) => {
    if ((0, exports.isResourceMapTwig)(entryValue) && (0, exports.isResourceMapTwig)(existValue)) {
        Object.keys(existValue).forEach((key) => {
            let val = (0, exports.selectAs)(existValue, key);
            if ('function' === typeof val) {
                (0, exports.insertEntry)(entryValue, val, key);
            }
            else if (Array.isArray(val)) {
                let targetArray = entryValue[key];
                val.forEach((elem, index) => {
                    if ('function' === typeof elem) {
                        targetArray[index] = elem;
                    }
                });
            }
            else if ((0, exports.isResourceMapTwig)(val)) {
                refillFunctionValues(entryValue[key], val);
            }
        });
    }
    return entryValue;
};
/**
 * マップの deep コピーを生成する。
 * @param existValue コピー元のマップ
 * @param refillFunctions コピー元の Function 型あるいは Function[] 型の値を生成したマップに登録する場合は true を指定する。デフォルト値は false
 */
const duplicateEntryValue = (existValue, refillFunctions = false) => {
    var dup = JSON.parse(JSON.stringify(existValue));
    refillFunctions && refillFunctionValues(dup, existValue);
    return dup;
};
exports.duplicateEntryValue = duplicateEntryValue;
/**
 * マップの二つの「枝」が等しいことを確認する。
 * @param valA マップの「枝」
 * @param valB マップのもうひとつの「枝」
 * @returns マップの二つの「枝」が等しい場合に true
 */
const equals = (valA, valB) => {
    return ((valA === null && valB === null)
        || ((typeof valA === typeof valB)
            && ((Array.isArray(valA) && Array.isArray(valB)
                && equalsEntryValueArray(valA, valB))
                || (!Array.isArray(valA) && !Array.isArray(valB) && (0, exports.isResourceMapLeaf)(valA) && valA === valB)
                || ((0, exports.isResourceMapTwig)(valA) && (0, exports.equalsEntryValue)(valA, valB)))));
};
/**
 * マップの二つの「枝」が配列型の場合に、それらが要素も含めて、等しいことを確認する。
 * @param arrayA マップの配列型の「枝」
 * @param arrayB マップのもうひとつの配列型の「枝」
 * @returns マップの二つの配列型の「枝」が等しい場合に true
 */
const equalsEntryValueArray = (arrayA, arrayB) => {
    return ((arrayA.length == arrayB.length)
        && arrayA.findIndex((elemA, index) => {
            return !equals(elemA, arrayB[index]);
        }) < 0);
};
/**
 * 二つのマップの「枝」が、それらに登録されているすべてのエントリも含めて、等しいことを確認する。
 * @param twigA マップの「枝」
 * @param twigB マップのもうひとつの「枝」
 * @returns 二つのマップの「枝」が、それらに登録されているすべてのエントリも含めて、等しい場合に true
 */
const equalsEntryValue = (twigA, twigB) => {
    if (Array.isArray(twigA) && Array.isArray(twigB)) {
        return equalsEntryValueArray(twigA, twigB);
    }
    if ((0, exports.isResourceMapTwig)(twigA) && (0, exports.isResourceMapTwig)(twigB)) {
        let [keysA, keysB] = [Object.keys(twigA), Object.keys(twigB)];
        return (keysA.length == keysB.length
            && (!Array.isArray(twigA) && !Array.isArray(twigB) && keysA.findIndex((key, index) => {
                try {
                    return !equals((0, exports.selectAs)(twigA, key), (0, exports.selectAs)(twigB, key));
                }
                catch (e) {
                    return true;
                }
            }) < 0));
    }
    return twigA === twigB;
};
exports.equalsEntryValue = equalsEntryValue;
/**
 * マップが空っぽであることを確認する。
 * @param aMap 確認する対象のマップ
 * @returns マップにエントリがひとつも登録されていない場合に true
 */
const isEmpty = (aMap) => {
    return (0, exports.equalsEntryValue)({}, aMap);
};
exports.isEmpty = isEmpty;
// 【...Utility】
// 【Manipulate twigs (move, link, copy)...】
/**
 * マップの「枝」を操作する。
 *
 * (1) move(), moveEntry() は、あるエントリの値のキーを変更する。
 *
 * (2) link(), linkEntry() は、あるエントリの値の shallow コピーで別のあらたなエントリを作成する。
 *
 * (3) copy(), copyEntry() は、あるエントリの値の deep コピーで別のあらたなエントリを作成する。
 * @param functionName 呼び出した関数の名称。
 * @param resourceMap 操作の対象とするマップ
 * @param makeLink あらたなエントリを作成する場合に true を指定する ((2), (3) が該当する)。
 * @param duplicateEntryValue あるエントリの値の deep コピーをあらたなエントリの値とする場合に true を指定する ((3) が該当する)。
 * @param deleteExistResourceKeys 既存のエントリを削除する場合に true を指定する ((1) が該当する)。
 * @param existResourceKeys 既存のあるエントリを指定するキーをコードの配列で指定する。
 * @param resourceKeys 更新/作成/変更する別のエントリのキーをコードの配列で指定する。
 * @param refillFunctions あるエントリの値のコピーにコピー元に登録されている関数もコピーする場合に true を指定する ((3) が該当する)。
 * @returns 「枝」の操作をした結果のマップ
 * @throws 以下の場合に例外を投げる。
 *
 * (1) エントリを指定するキーが指定されていない場合
 *
 * (2) 操作元になるエントリがマップに登録されていない場合
 *
 * (3) あらたなエントリを作成する場合 (makeLink が true) にあらたに作成するエントリがすでにマップに登録されている場合
 *
 * (4) 「枝」を実際に操作するための他の関数 (duplicateEntryValue(), update(), insert(), deleteIt()) で例外が発生した場合
 */
const manipulateTwigs = (functionName, resourceMap, makeLink, dupulicateExistValue, deleteExistResourceKeys, existResourceKeys, resourceKeys, refillFunctions = false) => {
    const exceptionName = exceptionMsg(functionName);
    if (existResourceKeys.length == 0 || resourceKeys.length == 0) {
        throw [exceptionName, resourceMap, makeLink, dupulicateExistValue, deleteExistResourceKeys, existResourceKeys, resourceKeys, refillFunctions,
            EXCEPTION_MSG_END
        ];
    }
    let [existIt, lastIndex, existValue] = (0, exports.search)(resourceMap, existResourceKeys);
    if (!existIt) {
        throw [exceptionName, resourceMap, makeLink, dupulicateExistValue, deleteExistResourceKeys, existResourceKeys, resourceKeys, refillFunctions,
            EXCEPTION_MSG_VARIABLES, existIt, lastIndex, existValue,
            EXCEPTION_MSG_END
        ];
    }
    let [foundIt, lastIndex2, existValue2] = (0, exports.search)(resourceMap, resourceKeys);
    if (foundIt && makeLink) {
        throw [exceptionName, resourceMap, makeLink, dupulicateExistValue, deleteExistResourceKeys, existResourceKeys, resourceKeys, refillFunctions,
            EXCEPTION_MSG_VARIABLES, existIt, lastIndex, existValue, foundIt, lastIndex2, existValue2,
            EXCEPTION_MSG_END
        ];
    }
    try {
        if (dupulicateExistValue) {
            existValue = (0, exports.duplicateEntryValue)(existValue, refillFunctions);
        }
        foundIt
            ? (0, exports.update)(resourceMap, existValue, resourceKeys)
            : (0, exports.insert)(resourceMap, existValue, resourceKeys);
        deleteExistResourceKeys &&
            (0, exports.deleteIt)(resourceMap, existResourceKeys);
        return resourceMap;
    }
    catch (e) {
        throw [exceptionName, resourceMap, makeLink, dupulicateExistValue, deleteExistResourceKeys, existResourceKeys, resourceKeys, refillFunctions,
            EXCEPTION_MSG_VARIABLES, existIt, lastIndex, existValue, foundIt, lastIndex2, existValue2,
            causedExceptionMsg(e)
        ];
    }
};
/**
 * あるエントリの値のキーを変更する。
 * @param resourceMap 操作の対象とするマップ
 * @param existResourceKeys 既存のあるエントリを指定するキーをコードの配列で指定する。
 * @param resourceKeys 変更する別のエントリのキーをコードの配列で指定する。
 * @param functionName 呼び出した関数の名称。デフォルト値は 'move'
 * @returns あるエントリの値のキーを変更した結果のマップ
 * @throws 以下の場合に例外を投げる。
 *
 * (1) エントリを指定するキーが指定されていない場合
 *
 * (2) 操作元になるエントリがマップに登録されていない場合
 *
 * (3) 「枝」を実際に操作するための他の関数 (update(), insert(), deleteIt()) で例外が発生した場合
 */
const move = (resourceMap, existResourceKeys, resourceKeys, functionName = 'move') => {
    return manipulateTwigs(functionName, resourceMap, false, false, true, existResourceKeys, resourceKeys);
};
exports.move = move;
/**
 * あるエントリの値のキーを変更する。
 * @param resourceMap 操作の対象とするマップ
 * @param existResourceKeys 既存のあるエントリを指定するキーをコードの配列で指定する。
 * @param resourceKeys 変更する別のエントリのキーをコードの可変引数で指定する。
 * @returns あるエントリの値のキーを変更した結果のマップ
 * @throws 以下の場合に例外を投げる。
 *
 * (1) エントリを指定するキーが指定されていない場合
 *
 * (2) 操作元になるエントリがマップに登録されていない場合
 *
 * (3) 「枝」を実際に操作するための他の関数 (update(), insert(), deleteIt()) で例外が発生した場合
 */
const moveEntry = (resourceMap, existResourceKeys, ...resourceKeys) => {
    return (0, exports.move)(resourceMap, existResourceKeys, resourceKeys, 'moveEntry');
};
exports.moveEntry = moveEntry;
/**
 * あるエントリの値の shallow コピーで別のあらたなエントリを作成する。
 *
 * @param resourceMap 操作の対象とするマップ
 * @param existResourceKeys 既存のあるエントリを指定するキーをコードの配列で指定する。
 * @param resourceKeys shallow コピーを作成する別のエントリのキーをコードの配列で指定する。
 * @param functionName 呼び出した関数の名称。デフォルト値は 'link'
 * @returns shallow コピーであらたなエントリを作成した結果のマップ
 * @throws 以下の場合に例外を投げる。
 *
 * (1) エントリを指定するキーが指定されていない場合
 *
 * (2) 操作元になるエントリがマップに登録されていない場合
 *
 * (3) あらたに作成するエントリがすでにマップに登録されている場合
 *
 * (4) 「枝」を実際に操作するための他の関数 (insert()) で例外が発生した場合
 */
const link = (resourceMap, existResourceKeys, resourceKeys, functionName = 'link') => {
    return manipulateTwigs(functionName, resourceMap, true, false, false, existResourceKeys, resourceKeys);
};
exports.link = link;
/**
 * あるエントリの値の shallow コピーで別のあらたなエントリを作成する。
 *
 * @param resourceMap 操作の対象とするマップ
 * @param existResourceKeys 既存のあるエントリを指定するキーをコードの配列で指定する。
 * @param resourceKeys shallow コピーを作成する別のエントリのキーをコードの可変引数で指定する。
 * @returns shallow コピーであらたなエントリを作成した結果のマップ
 * @throws 以下の場合に例外を投げる。
 *
 * (1) エントリを指定するキーが指定されていない場合
 *
 * (2) 操作元になるエントリがマップに登録されていない場合
 *
 * (3) あらたに作成するエントリがすでにマップに登録されている場合
 *
 * (4) 「枝」を実際に操作するための他の関数 (insert()) で例外が発生した場合
 */
const linkEntry = (resourceMap, existResourceKeys, ...resourceKeys) => {
    return (0, exports.link)(resourceMap, existResourceKeys, resourceKeys, 'linkEntry');
};
exports.linkEntry = linkEntry;
/**
 * あるエントリの値の deep コピーで別のあらたなエントリを作成する。
 * @param resourceMap 操作の対象とするマップ
 * @param existResourceKeys 既存のあるエントリを指定するキーをコードの配列で指定する。
 * @param resourceKeys 更新/作成/変更する別のエントリのキーをコードの配列で指定する。
 * @param refillFunctions あるエントリの値のコピーにコピー元に登録されている関数もコピーする場合に true を指定する。デフォルト値は false
 * @param functionName 呼び出した関数の名称。デフォルト値は 'copy'
 * @returns deep コピーであらたなエントリを作成した結果のマップ
 * @throws 以下の場合に例外を投げる。
 *
 * (1) エントリを指定するキーが指定されていない場合
 *
 * (2) 操作元になるエントリがマップに登録されていない場合
 *
 * (3) あらたに作成するエントリがすでにマップに登録されている場合
 *
 * (4) 「枝」を実際に操作するための他の関数 (duplicateEntryValue(), insert()) で例外が発生した場合
 */
const copy = (resourceMap, existResourceKeys, resourceKeys, refillFunctions = false, functionName = 'copy') => {
    return manipulateTwigs(functionName, resourceMap, true, true, false, existResourceKeys, resourceKeys, refillFunctions);
};
exports.copy = copy;
/**
 * あるエントリの値の deep コピーで別のあらたなエントリを作成する。
 * @param resourceMap 操作の対象とするマップ
 * @param refillFunctions あるエントリの値のコピーにコピー元に登録されている関数もコピーする場合に true を指定する。
 * @param existResourceKeys 既存のあるエントリを指定するキーをコードの配列で指定する。
 * @param resourceKeys 更新/作成/変更する別のエントリのキーをコードの可変引数で指定する。
 * @returns deep コピーであらたなエントリを作成した結果のマップ
 * @throws 以下の場合に例外を投げる。
 *
 * (1) エントリを指定するキーが指定されていない場合
 *
 * (2) 操作元になるエントリがマップに登録されていない場合
 *
 * (3) あらたに作成するエントリがすでにマップに登録されている場合
 *
 * (4) 「枝」を実際に操作するための他の関数 (duplicateEntryValue(), insert()) で例外が発生した場合
 */
const copyEntry = (resourceMap, refillFunctions, existResourceKeys, ...resourceKeys) => {
    return (0, exports.copy)(resourceMap, existResourceKeys, resourceKeys, refillFunctions, 'copyEntry');
};
exports.copyEntry = copyEntry;
// 【...Manipulate twigs (move, link, copy)】
//【End of program】
//【Begin of tests】
//【ResourceMap の動作確認】
// ------------------------------------------------------------------------
// 【テスト】コードを表示文字列に変換する機能 ResourceMap と sel*() は、
//      以下のようにしてマップとコードの定数を定義して利用する。
// ------------------------------------------------------------------------
/** コード 1 number → number */
const CODE_1 = 1;
/** コード 2 number → string */
const CODE_2 = 10;
/** コード 3 string → number */
const CODE_3 = 'hundred';
/** コード 4 string → string */
const CODE_4 = '1000';
/** コード 5 number → ResourceMap（string → number） */
const CODE_5 = 2;
/** コード 5-1 string → number */
const CODE_5_1 = 'abc';
/** コード 6 string → ResourceMap（numner →　ResourceMap（string → string）） */
const CODE_6 = 'three';
/** コード 6-1 numner →　ResourceMap（string → string） */
const CODE_6_1 = 3;
/** コード 6-1-1 string → string */
const CODE_6_1_1 = '三';
/** コード 6-1-2 string → string */
const CODE_6_1_2 = 'III';
/** コード 7 number → object */
const CODE_7 = 4;
/** コード 7 string → Function[] */
const CODE_7_MULTIPLISE = 'multiplise';
/** コード 7 string → Function */
const CODE_7_ADDING = 'adding';
/** コード 7 string → string */
const CODE_7_FACT = 'reality is a dream.';
/** コード 8 string → object[] */
const CODE_8 = 'five';
/** コード 8-1 string → string[] */
const CODE_8_1 = 'eyes';
/** コード 9 number → number[] */
const CODE_9 = 12;
/** コード 10 string → string */
const CODE_10 = 'yet';
/**
 * 数値を 10 倍する。
 * @param n 数値を指定する。
 * @returns 数値を 10 倍した結果。
 */
const mul10 = (n) => { return 10 * n; };
/**
 * 数値を自乗する。
 * @param n 数値を指定する。
 * @returns 数値の自乗。
 */
const square = (n) => { return n ** 2; };
/**
 * 数値に 3 を足す。
 * @param n 数値を指定する。
 * @returns 数値に 3 を足した結果。
 */
const add3 = (n) => { return 3 + n; };
//XXX 何回か繰り返して性能を評価する。
const PERFORM_ITERATE_CNT = 1;
for (let perform = 0; perform < PERFORM_ITERATE_CNT; perform++) {
    /** コードとその値のマップの例
     * @type {ResourceMap}
     */
    let sampleMap = {
        1: 0,
        10: 'ten',
        'hundred': 100,
        '1000': 'thousand',
        2: { 'abc': 20000 },
        'three': { 3: { '三': '参拾萬', 'III': 300000 } },
        4: { adding: add3, multiplise: [mul10, square],
            sound: 'nice', color: 'beautiful', smell: 'good', touch: 'fine', taste: 'sweet',
            5: 'five senses', 'reality is a dream.': true
        },
        'five': { 'eyes': { nations: ['us', 'ca', 'gb', 'au', 'nz'],
                isUk: [false, true, true, true, true]
            }
        },
        12: [9]
    };
    //  コードとその値のマップの例 sampleMap とコードの定数 CODE_1 ～ CODE_6_1_1 を使用して ResourceMap の動作を確認している。
    const ELASPED_TIME_ALL = 'ELAPSED_TIME_ALL';
    const ELAPSED_TIME_TEST1 = 'ELAPSED_TIME_TEST1';
    const ELAPSED_TIME_TEST2 = 'ELAPSED_TIME_TEST2';
    const ELAPSED_TIME_TEST3 = 'ELAPSED_TIME_TEST3';
    const ELAPSED_TIME_TEST4 = 'ELAPSED_TIME_TEST4';
    const ELAPSED_TIME_TEST5 = 'ELAPSED_TIME_TEST5';
    const ELAPSED_TIME_XXX = 'ELAPSED_TIME_XXX';
    let testCnt = 0;
    console.time(ELASPED_TIME_ALL); //XXX show elasped time
    console.log('TEST(START)...');
    console.time(ELAPSED_TIME_TEST1); //XXX show elasped time
    console.log('TEST: sampleMap initial value is...');
    console.log(sampleMap);
    console.log('TEST(1):search(): ' + ++testCnt);
    {
        let [foundIt, lastIndex, result] = (0, exports.search)(sampleMap, []);
        console.log('foundIt is ' + foundIt + '. lastIndex is ' + lastIndex + '. result is ' + result);
    }
    console.log('TEST(1):selectAs<EntryValue>(): ' + ++testCnt);
    try {
        (0, exports.selectAs)(sampleMap);
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(1):search(): ' + ++testCnt);
    {
        let [foundIt, lastIndex, result] = (0, exports.search)(sampleMap, ['']);
        console.log('foundIt is ' + foundIt + '. lastIndex is ' + lastIndex + '. result is ' + result);
    }
    console.log('TEST(1):selectAs<EntryValue>(): ' + ++testCnt);
    try {
        (0, exports.selectAs)(sampleMap, '');
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(1):search(): ' + ++testCnt);
    {
        let [foundIt, lastIndex, result] = (0, exports.search)(sampleMap, [CODE_1]);
        console.log('foundIt is ' + foundIt + '. lastIndex is ' + lastIndex + '. result is ' + result);
    }
    console.log('TEST(1):selectAs<EntryValue>(): ' + ++testCnt);
    console.log((0, exports.selectAs)(sampleMap, CODE_1));
    console.log('TEST(1):search(): ' + ++testCnt);
    {
        let [foundIt, lastIndex, result] = (0, exports.search)((0, exports.selectAs)(sampleMap, CODE_2), [CODE_3]);
        console.log('foundIt is ' + foundIt + '. lastIndex is ' + lastIndex + '. result is ' + result);
    }
    console.log('TEST(1):search(): ' + ++testCnt);
    {
        let [foundIt, lastIndex, result] = (0, exports.search)(sampleMap, ['マップ sampleMap に定義されていないコード']);
        console.log('foundIt is ' + foundIt + '. lastIndex is ' + lastIndex + '. result is ' + result);
    }
    console.log('TEST(1):selectAs<number|string|EntryValue>(): ' + ++testCnt);
    console.log((0, exports.selectAs)(sampleMap, CODE_1)
        + ', ' + (0, exports.selectAs)(sampleMap, CODE_2)
        + ', ' + (0, exports.selectAs)(sampleMap, CODE_3)
        + ', ' + (0, exports.selectAs)(sampleMap, CODE_4)
        + ', ' + (0, exports.selectAs)((0, exports.selectAs)(sampleMap, CODE_5), CODE_5_1)
        + ', ' + (0, exports.selectAs)((0, exports.selectAs)((0, exports.selectAs)(sampleMap, CODE_6), CODE_6_1), CODE_6_1_1)
        + ', ' + (0, exports.selectAs)((0, exports.selectAs)((0, exports.selectAs)(sampleMap, CODE_6), CODE_6_1), CODE_6_1_2));
    console.log('TEST(1):selectAs<number|string|EntryValue>(): ' + ++testCnt);
    console.log((0, exports.selectAs)(sampleMap, CODE_1)
        + ', ' + (0, exports.selectAs)(sampleMap, CODE_2)
        + ', ' + (0, exports.selectAs)(sampleMap, CODE_3)
        + ', ' + (0, exports.selectAs)(sampleMap, CODE_4)
        + ', ' + (0, exports.selectAs)(sampleMap, CODE_5, CODE_5_1)
        + ', ' + (0, exports.selectAs)(sampleMap, CODE_6, CODE_6_1, CODE_6_1_1)
        + ', ' + (0, exports.selectAs)(sampleMap, CODE_6, CODE_6_1, CODE_6_1_2));
    console.log('TEST(1):selectAs<EntryValue>(): ' + ++testCnt);
    console.log((0, exports.selectAs)(sampleMap, CODE_7));
    console.log('TEST(1):selectAs<EntryValue>(): ' + ++testCnt);
    console.log((0, exports.selectAs)(sampleMap, CODE_7, 5));
    console.log('TEST(1):selectAs<ResouceMap>(): ' + ++testCnt);
    console.log(((0, exports.selectAs)(sampleMap, CODE_7)).multiplise[0]);
    console.log('TEST(1):selectAs<ResouceMap>(): ' + ++testCnt);
    console.log((((0, exports.selectAs)(sampleMap, CODE_7)).multiplise[0])(3));
    console.log('TEST(1):selectAs<Function[]>(): ' + ++testCnt);
    console.log((0, exports.selectAs)(sampleMap, CODE_7, CODE_7_MULTIPLISE)[1]);
    console.log('TEST(1):selectAs<Function[]>(): ' + ++testCnt);
    console.log(((0, exports.selectAs)(sampleMap, CODE_7, CODE_7_MULTIPLISE)[1])(3));
    console.log('TEST(1):selectAs<Function>(): ' + ++testCnt);
    console.log((0, exports.selectAs)(sampleMap, CODE_7, CODE_7_ADDING));
    console.log('TEST(1):selectAs<Function>(): ' + ++testCnt);
    console.log(((0, exports.selectAs)(sampleMap, CODE_7, CODE_7_ADDING))(4));
    console.log('TEST(1):selectAs<boolean>(): ' + ++testCnt);
    console.log((0, exports.selectAs)(sampleMap, CODE_7, CODE_7_FACT));
    console.log('TEST(1):selectAs<EntryValue>(): ' + ++testCnt);
    console.log((0, exports.selectAs)(sampleMap, CODE_8));
    console.log('TEST(1):selectAs<EntryValue>(): ' + ++testCnt);
    console.log((0, exports.selectAs)(sampleMap, CODE_8, CODE_8_1));
    console.log('TEST(1):selectAs<object[]>(): ' + ++testCnt);
    console.log(((0, exports.selectAs)(sampleMap, CODE_8, CODE_8_1, 'nations'))[0]);
    console.log('TEST(1):selectAs<boolean[]>(): ' + ++testCnt);
    console.log(((0, exports.selectAs)(sampleMap, CODE_8, CODE_8_1, 'isUk'))[0]);
    console.log('TEST(1):selectAs<string[]>(): ' + ++testCnt);
    console.log(((0, exports.selectAs)(sampleMap, CODE_8, CODE_8_1, 'nations'))[2]);
    console.log('TEST(1):selectAs<string[]>(): ' + ++testCnt);
    console.log(((0, exports.selectAs)(sampleMap, CODE_8, CODE_8_1, 'isUk'))[2]);
    console.log('TEST(1):selectAs<EntryValue>(): ' + ++testCnt);
    console.log((0, exports.selectAs)(sampleMap, CODE_9));
    console.log('TEST(1):selectAs<EntryValue>(): ' + ++testCnt);
    console.log((0, exports.selectAs)(sampleMap, CODE_9) === (0, exports.selectAs)(sampleMap, 12));
    console.log('TEST(1):selectAs<EntryValue>(): ' + ++testCnt);
    console.log((0, exports.selectAs)(sampleMap, CODE_9) == (0, exports.selectAs)(sampleMap, 12));
    console.log('TEST(1):selectAs<number[]>(): ' + ++testCnt);
    console.log((0, exports.selectAs)(sampleMap, CODE_9) === ([9]));
    console.log('TEST(1):selectAs<number[]>(): ' + ++testCnt);
    console.log((0, exports.selectAs)(sampleMap, CODE_9) == ([9]));
    console.log('TEST(1):selectAs<number[]>(): ' + ++testCnt);
    {
        const nums = [9];
        console.log((0, exports.selectAs)(sampleMap, CODE_9) === nums);
        console.log((0, exports.selectAs)(sampleMap, CODE_9) == nums);
    }
    console.log('TEST(1):insertEntry(),selectAs<EntryValue>():  ' + ++testCnt);
    console.log((0, exports.selectAs)((0, exports.insertEntry)(sampleMap, 'another', CODE_10), CODE_10));
    console.log('TEST:console/log(sampleMap): ' + ++testCnt);
    console.log(sampleMap);
    console.log('TEST(1):deleteEntry(),insertEntry(),selectAs<number[]>():  ' + ++testCnt);
    console.log((0, exports.selectAs)((0, exports.insertEntry)((0, exports.deleteEntry)(sampleMap, CODE_10), [2, 3, 5, 7, 11, 13, 17, 19, 23, 29], CODE_10), CODE_10)[3]);
    console.log('TEST(1):console/log(sampleMap): ' + ++testCnt);
    console.log(sampleMap);
    console.log('TEST(1):deleteEntry(): ' + ++testCnt);
    console.log((0, exports.deleteEntry)(sampleMap, CODE_10));
    console.log('TEST(1):deleteEntry(), selectAs<EntryValue>(): ' + ++testCnt);
    {
        try {
            (0, exports.selectAs)((0, exports.deleteEntry)(sampleMap, CODE_10), CODE_10);
        }
        catch (e) {
            console.log(e);
        }
    }
    console.log('TEST(1):cleanUp(): ' + ++testCnt);
    console.log((0, exports.cleanUp)(sampleMap, true, '存在しないコード'));
    console.log('TEST(1):clean(): ' + ++testCnt);
    console.log((0, exports.clean)(sampleMap, true, [CODE_8, CODE_8_1]));
    console.log('TEST(1):cleanUp(): ' + ++testCnt);
    console.log((0, exports.cleanUp)(sampleMap, true));
    console.log('TEST(1):cleanUp(): ' + ++testCnt);
    console.log((0, exports.cleanUp)(sampleMap, false));
    console.log('TEST(1):select<number>(): ' + ++testCnt);
    try {
        console.log((0, exports.select)(sampleMap, [CODE_1]));
    }
    catch (e) {
        console.log(e);
    }
    console.timeEnd(ELAPSED_TIME_TEST1); //XXX show elasped time
    //XXX CRUD...
    console.time(ELAPSED_TIME_TEST2); //XXX show elasped time
    //XXX コードとその値を定義したマップを新規に作成する。
    console.time(ELAPSED_TIME_XXX); //XXX show elasped time
    console.log('TEST(2):create(): ' + ++testCnt);
    let sampleMap2 = (0, exports.create)();
    console.log(sampleMap2);
    console.timeEnd(ELAPSED_TIME_XXX); //XXX show elasped time
    //XXX insert
    console.time(ELAPSED_TIME_XXX); //XXX show elasped time
    console.log('TEST(2):insertEntry(): ' + ++testCnt);
    try {
        console.log((0, exports.insertEntry)(sampleMap2, []));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):insertEntry(): ' + ++testCnt);
    console.log((0, exports.insertEntry)(sampleMap2, 'がじづべぽ', '平仮名'));
    console.log('TEST(2):insert(): ' + ++testCnt);
    try {
        console.log((0, exports.insert)(sampleMap2, 'がじづべぽ', ['平仮名']));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):insert(): ' + ++testCnt);
    console.log(sampleMap2);
    console.log((0, exports.insert)(sampleMap2, 'ｶﾞｼﾞﾂﾞﾍﾞﾎﾟ', ['ｶﾀ', 'ｶﾅ']));
    console.log('TEST(2):insertEntry(): ' + ++testCnt);
    try {
        console.log((0, exports.insertEntry)(sampleMap2, 'ｶﾞｼﾞﾂﾞﾍﾞﾎﾟ', 'ｶﾀ', 'ｶﾅ'));
    }
    catch (e) {
        console.log(e);
    }
    console.timeEnd(ELAPSED_TIME_XXX); //XXX show elasped time
    //XXX delete
    console.time(ELAPSED_TIME_XXX); //XXX show elasped time
    console.log('TEST(2):deleteEntry(): ' + ++testCnt);
    try {
        console.log((0, exports.deleteEntry)(sampleMap2));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):deleteIt(): ' + ++testCnt);
    try {
        console.log((0, exports.deleteIt)(sampleMap2, []));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):deleteEntry(): ' + ++testCnt);
    console.log((0, exports.deleteEntry)(sampleMap2, '平仮名'));
    console.log('TEST(2):delete - selectAs(): ' + ++testCnt);
    const kanaKanaValue = (0, exports.selectAs)(sampleMap2, 'ｶﾀ', 'ｶﾅ');
    console.log(kanaKanaValue);
    console.log('TEST(2):deleteEntry(): ' + ++testCnt);
    console.log((0, exports.deleteEntry)(sampleMap2, 'ｶﾀ'));
    console.log('TEST(2):deleteEntry(): ' + ++testCnt);
    console.log((0, exports.insertEntry)(sampleMap2, kanaKanaValue, 'ｶﾀ', 'ｶﾅ'));
    console.log('TEST(2):deleteEntry(): ' + ++testCnt);
    console.log((0, exports.deleteEntry)(sampleMap2, 'ｶﾀ', 'ｶﾅ'));
    console.timeEnd(ELAPSED_TIME_XXX); //XXX show elasped time
    //XXX update
    console.time(ELAPSED_TIME_XXX); //XXX show elasped time
    console.log('TEST(2):update(): ' + ++testCnt);
    try {
        console.log((0, exports.update)(sampleMap2, 'カタカナ', []));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):updateEntry(): ' + ++testCnt);
    try {
        console.log((0, exports.updateEntry)(sampleMap2, 'カタカナ'));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):update(): ' + ++testCnt);
    try {
        console.log((0, exports.update)(sampleMap2, 'カタカナ', ['ｶﾅ']));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):updateEntry(): ' + ++testCnt);
    try {
        console.log((0, exports.updateEntry)(sampleMap2, 'カタカナ', 'ｶﾅ'));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):update(): ' + ++testCnt);
    try {
        console.log((0, exports.update)(sampleMap2, 'カタカナ', ['ｶﾀ', 'ｶﾅ']));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):updateEntry(): ' + ++testCnt);
    try {
        console.log((0, exports.updateEntry)(sampleMap2, 'カタカナ', 'ｶﾀ', 'ｶﾅ'));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):update(): ' + ++testCnt);
    console.log((0, exports.update)(sampleMap2, { 'ｶﾅ': 'katakana' }, ['ｶﾀ']));
    console.log('TEST(2):update(): ' + ++testCnt);
    console.log((0, exports.update)(sampleMap2, { 'ｶﾅ': 'KATAKANA' }, ['ｶﾀ']));
    console.log('TEST(2):update(): ' + ++testCnt);
    console.log((0, exports.update)(sampleMap2, 'カタカナ', ['ｶﾀ', 'ｶﾅ']));
    console.log('TEST(2):updateEntry(): ' + ++testCnt);
    console.log((0, exports.updateEntry)(sampleMap2, '片仮名', 'ｶﾀ', 'ｶﾅ'));
    console.timeEnd(ELAPSED_TIME_XXX); //XXX show elasped time
    //XXX merge
    console.time(ELAPSED_TIME_XXX); //XXX show elasped time
    console.log('TEST(2):merge(): ' + ++testCnt);
    let m = (0, exports.create)();
    try {
        const add1 = (n) => { return n + 1; };
        (0, exports.insert)(m, true, ['bool']);
        (0, exports.insert)(m, add1, ['func']);
        (0, exports.insert)(m, 1, ['num']);
        (0, exports.insert)(m, 'abc', ['str']);
        (0, exports.insert)(m, { 'x': 0, 'y': 1 }, ['sub-map']);
        try {
            console.log((0, exports.merge)(m, 2, []));
        }
        catch (e) {
            console.log(e);
        }
        try {
            console.log((0, exports.merge)(m, 2, ['bool', 'merged']));
        }
        catch (e) {
            console.log(e);
        }
        try {
            console.log((0, exports.merge)(m, 2, ['func', 'merged']));
        }
        catch (e) {
            console.log(e);
        }
        try {
            console.log((0, exports.merge)(m, 2, ['num', 'merged']));
        }
        catch (e) {
            console.log(e);
        }
        try {
            console.log((0, exports.merge)(m, 2, ['str', 'merged']));
        }
        catch (e) {
            console.log(e);
        }
        try {
            console.log((0, exports.merge)(m, 2, ['sub-map', 'z']));
        }
        catch (e) {
            console.log(e);
        }
        try {
            console.log((0, exports.merge)(m, { 'あ': 'い' }, ['sub-map', 'merged']));
        }
        catch (e) {
            console.log(e);
        }
        try {
            console.log((0, exports.merge)(m, (0, exports.duplicateEntryValue)(m), ['sub-map', 'merged']));
        }
        catch (e) {
            console.log(e);
        }
    }
    catch (e) {
        console.trace();
        console.log('XXX:DBG setting for mgr', e);
    }
    finally {
        (0, exports.clean)(m, true, []);
    }
    console.log('TEST(2):mergeEntry(): ' + ++testCnt);
    try {
        console.log((0, exports.mergeEntry)(sampleMap2, (0, exports.selectAs)(sampleMap2, 'ｶﾀ', 'ｶﾅ')));
    }
    catch (e) {
        console.log(e);
    }
    try {
        console.log((0, exports.mergeEntry)(sampleMap2, (0, exports.selectAs)(sampleMap2, 'ｶﾀ', 'ｶﾅ'), '平仮名'));
    }
    catch (e) {
        console.log(e);
    }
    try {
        console.log((0, exports.mergeEntry)(sampleMap2, (0, exports.selectAs)(sampleMap2, 'ｶﾀ', 'ｶﾅ'), '平仮名', '例外を起こす'));
    }
    catch (e) {
        console.log(e);
    }
    try {
        console.log((0, exports.mergeEntry)(sampleMap2, 'ひらがな', 'ｶﾀ', 'ｶﾅ'));
    }
    catch (e) {
        console.log(e);
    }
    try {
        console.log((0, exports.mergeEntry)(sampleMap2, (0, exports.select)(sampleMap2, ['ｶﾀ', 'ｶﾅ']), '平仮名'));
    }
    catch (e) {
        console.log(e);
    }
    console.timeEnd(ELAPSED_TIME_XXX); //XXX show elasped time
    console.timeEnd(ELAPSED_TIME_TEST2); //XXX show elasped time
    //XXX ...CRUD
    //XXX manipulateTwigs...
    console.time(ELAPSED_TIME_TEST3); //XXX show elasped time
    //XXX link
    console.time(ELAPSED_TIME_XXX); //XXX show elasped time
    console.log('TEST(2):link(): ' + ++testCnt);
    console.log((0, exports.update)(sampleMap2, 'カタカナ', ['ｶﾀ', 'ｶﾅ']));
    try {
        console.log((0, exports.link)(sampleMap2, [999], []));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):linkEntry(): ' + ++testCnt);
    try {
        console.log((0, exports.linkEntry)(sampleMap2, [999]));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):link(): ' + ++testCnt);
    console.log((0, exports.link)(sampleMap2, ['ｶﾀ'], [999]));
    console.log('TEST(2):linkEntry(): ' + ++testCnt);
    try {
        console.log((0, exports.linkEntry)(sampleMap2, ['ｶﾀ'], 999));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):link() した値を update(): ' + ++testCnt);
    console.log((0, exports.update)(sampleMap2, 'カナ', ['ｶﾀ']));
    console.timeEnd(ELAPSED_TIME_XXX); //XXX show elasped time
    //XXX move
    console.time(ELAPSED_TIME_XXX); //XXX show elasped time
    console.log('TEST(2):move(): ' + ++testCnt);
    try {
        console.log((0, exports.move)(sampleMap2, [888], []));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):moveEntry(): ' + ++testCnt);
    try {
        console.log((0, exports.moveEntry)(sampleMap2, [888]));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):move(): ' + ++testCnt);
    try {
        console.log((0, exports.move)(sampleMap2, [999], []));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):moveEntry(): ' + ++testCnt);
    try {
        console.log((0, exports.moveEntry)(sampleMap2, [999]));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):move(): ' + ++testCnt);
    try {
        console.log((0, exports.move)(sampleMap2, [999], [888, 'moved', 999]));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):moveEntry(): ' + ++testCnt);
    try {
        console.log((0, exports.moveEntry)(sampleMap2, [999], 888, 'moved', 999));
    }
    catch (e) {
        console.log(e);
    }
    console.timeEnd(ELAPSED_TIME_XXX);
    console.time(ELAPSED_TIME_XXX); //XXX show elasped time
    console.timeEnd(ELAPSED_TIME_XXX); //XXX show elasped time
    //XXX copy
    console.time(ELAPSED_TIME_XXX); //XXX show elasped time
    console.log('TEST(2):copy(): ' + ++testCnt);
    try {
        console.log((0, exports.copy)(sampleMap2, [777], []));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):copyEntry(): ' + ++testCnt);
    try {
        console.log((0, exports.copyEntry)(sampleMap2, false, [777]));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):copy(): ' + ++testCnt);
    try {
        console.log((0, exports.copy)(sampleMap2, [888], []));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):copyEntry(): ' + ++testCnt);
    try {
        console.log((0, exports.copyEntry)(sampleMap2, false, [888]));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):copy(): ' + ++testCnt);
    try {
        console.log((0, exports.copy)(sampleMap2, [888], ['copied'], true));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):copyEntry(): ' + ++testCnt);
    try {
        console.log((0, exports.copyEntry)(sampleMap2, true, [888], 'copied'));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):insert(): ' + ++testCnt);
    console.log((0, exports.insert)(sampleMap2, [mul10, add3], ['copied', '関数の配列']));
    try {
        console.log((0, exports.selectAs)(sampleMap2, 'copied', '関数の配列')[0](11));
    }
    catch (e) {
        console.log(e);
    }
    try {
        console.log((0, exports.selectAs)(sampleMap2, 'copied', '関数の配列')[1](11));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(2):copy(), updateEntry(): ' + ++testCnt);
    try {
        console.log((0, exports.copy)(sampleMap2, ['copied'], ['コピーしたcopied'], true));
    }
    catch (e) {
        console.log(e);
    }
    (0, exports.updateEntry)(sampleMap2, { deep: 'copied' }, 'コピーしたcopied', 'moved');
    console.log((0, exports.selectAs)(sampleMap2, 'copied'));
    console.log((0, exports.selectAs)(sampleMap2, 'コピーしたcopied'));
    console.log('TEST(2):selectAs<Function[]>(): ' + ++testCnt);
    (0, exports.selectAs)(sampleMap2, 'コピーしたcopied', '関数の配列')[1] = square;
    try {
        console.log((0, exports.selectAs)(sampleMap2, 'copied', '関数の配列')[0](11));
    }
    catch (e) {
        console.log(e);
    }
    try {
        console.log((0, exports.selectAs)(sampleMap2, 'コピーしたcopied', '関数の配列')[0](11));
    }
    catch (e) {
        console.log(e);
    }
    try {
        console.log((0, exports.selectAs)(sampleMap2, 'copied', '関数の配列')[1](11));
    }
    catch (e) {
        console.log(e);
    }
    try {
        console.log((0, exports.selectAs)(sampleMap2, 'コピーしたcopied', '関数の配列')[1](11));
    }
    catch (e) {
        console.log(e);
    }
    console.timeEnd(ELAPSED_TIME_XXX); //XXX show elasped time
    //XXX ...manipulateTwigs
    console.timeEnd(ELAPSED_TIME_TEST3);
    //XXX utility...
    console.time(ELAPSED_TIME_TEST4);
    console.time(ELAPSED_TIME_XXX); //XXX show elasped time
    //XXX toEntryKeys
    console.log('TEST(3):toEntryKeys(): ' + ++testCnt);
    const notResourceKeyPath = 'an, array, of, EntryKey';
    try {
        console.log((0, exports.toEntryKeys)(notResourceKeyPath));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(3):toEntryKeys(): ' + ++testCnt);
    const notResourceKeyPath2 = '[888]';
    try {
        console.log((0, exports.toEntryKeys)(notResourceKeyPath2));
    }
    catch (e) {
        console.log(e);
    }
    try {
        console.log((0, exports.select)(sampleMap2, (0, exports.toEntryKeys)(notResourceKeyPath2)));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(3):toEntryKeys(): ' + ++testCnt);
    const resourceKeyPath = '"an", "array", "of", "EntryKey"';
    const resourceKeys = (0, exports.toEntryKeys)(resourceKeyPath);
    try {
        console.log(resourceKeys);
    }
    catch (e) {
        console.log(e);
    }
    try {
        console.log((0, exports.select)(sampleMap2, resourceKeys));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(3):toEntryKeys(): ' + ++testCnt);
    console.log((0, exports.insert)(sampleMap2, ["EntryKey", 'の', '配列'], resourceKeys));
    try {
        console.log((0, exports.toEntryKeys)(resourceKeyPath));
    }
    catch (e) {
        console.log(e);
    }
    try {
        console.log((0, exports.select)(sampleMap2, resourceKeys));
    }
    catch (e) {
        console.log(e);
    }
    console.timeEnd(ELAPSED_TIME_XXX); //XXX show elasped time
    //XXX reverse
    console.time(ELAPSED_TIME_XXX); //XXX show elasped time
    console.log('TEST(3):reverse(): ' + ++testCnt);
    try {
        console.log((0, exports.reverse)(sampleMap2));
    }
    catch (e) {
        console.log(e);
    }
    let singleWidthToWideWidthChar = {
        0: '０',
        1: '１',
        2: '２',
        '7': '７',
        '8': '８',
        '9': '９',
        '+': '＋',
        '-': '－',
        '*': '×',
        '/': '÷',
        'A': 'Ａ',
        'B': 'Ｂ',
        'C': 'Ｃ',
        'x': 'ｘ',
        'y': 'ｙ',
        'z': 'ｚ',
        'ｶﾞ': 'ガ',
        'ｻﾞ': 'ザ',
    };
    console.log((0, exports.updateEntry)(singleWidthToWideWidthChar, 'ガ', 'ｻﾞ'));
    try {
        console.log((0, exports.reverse)(singleWidthToWideWidthChar));
    }
    catch (e) {
        console.log(e);
    }
    (0, exports.updateEntry)(singleWidthToWideWidthChar, 'ザ', 'ｻﾞ');
    console.log('TEST(3):reverse(): ' + ++testCnt);
    console.log(singleWidthToWideWidthChar);
    let wideWidthToSingleWidthChar = (0, exports.reverse)(singleWidthToWideWidthChar);
    console.log(wideWidthToSingleWidthChar);
    console.timeEnd(ELAPSED_TIME_XXX); //XXX show elasped time
    //XXX oneToOne
    console.time(ELAPSED_TIME_XXX); //XXX show elasped time
    console.log('TEST(3):oneToOne(): ' + ++testCnt);
    try {
        console.log('oneToOne(singleWidthToWideWidthChar, sampleMap2):', (0, exports.oneToOne)(singleWidthToWideWidthChar, sampleMap2));
    }
    catch (e) {
        console.log(e);
    }
    try {
        console.log('oneToOne(singleWidthToWideWidthChar, wideWidthToSingleWidthChar):', (0, exports.oneToOne)(singleWidthToWideWidthChar, wideWidthToSingleWidthChar));
    }
    catch (e) {
        console.log(e);
    }
    console.timeEnd(ELAPSED_TIME_XXX); //XXX show elasped time
    //XXX duplicateEntryValue
    console.time(ELAPSED_TIME_XXX); //XXX show elasped time
    console.log('TEST(3):duplicateEntryValue(): ' + ++testCnt);
    const rootOfOrigin = "duplicateTest";
    try {
        let rootOfPath = rootOfOrigin;
        let [pathOfSquare, pathOfAddAndMul] = ['"' + rootOfPath + '", "自乗"', '"' + rootOfPath + '", "加算と乗算"'];
        (0, exports.insert)(sampleMap2, square, (0, exports.toEntryKeys)(pathOfSquare));
        (0, exports.insert)(sampleMap2, [add3, mul10], (0, exports.toEntryKeys)(pathOfAddAndMul));
        (0, exports.insert)(sampleMap2, true, [rootOfOrigin, 'bool']);
        (0, exports.insert)(sampleMap2, 0, [rootOfOrigin, 'number']);
        (0, exports.insert)(sampleMap2, 'abc', [rootOfOrigin, 'string']);
        console.log(rootOfPath
            + ':自乗=' + (0, exports.select)(sampleMap2, (0, exports.toEntryKeys)(pathOfSquare))(5)
            + ':加算=' + (0, exports.select)(sampleMap2, (0, exports.toEntryKeys)(pathOfAddAndMul))[0](5)
            + ':乗算=' + (0, exports.select)(sampleMap2, (0, exports.toEntryKeys)(pathOfAddAndMul))[1](5));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(3):duplicateEntryValue(): ' + ++testCnt);
    try {
        let rootOfPath = "duplicated refilled functions";
        let [pathOfSquare, pathOfAddAndMul] = ['"' + rootOfPath + '", "自乗"', '"' + rootOfPath + '", "加算と乗算"'];
        console.log((0, exports.insertEntry)(sampleMap2, (0, exports.duplicateEntryValue)((0, exports.select)(sampleMap2, [rootOfOrigin]), true), rootOfPath));
        (0, exports.update)(sampleMap2, false, [rootOfPath, 'bool']);
        (0, exports.update)(sampleMap2, 1, [rootOfPath, 'number']);
        (0, exports.update)(sampleMap2, 'ABC', [rootOfPath, 'string']);
        console.log(rootOfPath
            + ':自乗=' + (0, exports.select)(sampleMap2, (0, exports.toEntryKeys)(pathOfSquare))(5)
            + ':加算=' + (0, exports.select)(sampleMap2, (0, exports.toEntryKeys)(pathOfAddAndMul))[0](5)
            + ':乗算=' + (0, exports.select)(sampleMap2, (0, exports.toEntryKeys)(pathOfAddAndMul))[1](5));
    }
    catch (e) {
        console.log(e);
    }
    console.log('TEST(3):duplicateEntryValue(): ' + ++testCnt);
    try {
        let rootOfPath = "duplicated without functions";
        let [pathOfSquare, pathOfAddAndMul] = ['"' + rootOfPath + '", "自乗"', '"' + rootOfPath + '", "加算と乗算"'];
        console.log((0, exports.insertEntry)(sampleMap2, (0, exports.duplicateEntryValue)((0, exports.select)(sampleMap2, [rootOfOrigin])), rootOfPath));
        try {
            console.log(rootOfPath
                + ':自乗=' + (0, exports.select)(sampleMap2, (0, exports.toEntryKeys)(pathOfSquare))(5));
        }
        catch (e) {
            console.log(e);
        }
        console.log(rootOfPath
            + ':加算=' + (0, exports.select)(sampleMap2, (0, exports.toEntryKeys)(pathOfAddAndMul))[0](5)
            + ':乗算=' + (0, exports.select)(sampleMap2, (0, exports.toEntryKeys)(pathOfAddAndMul))[1](5));
    }
    catch (e) {
        console.log(e);
    }
    console.timeEnd(ELAPSED_TIME_XXX); //XXX show elasped time
    //XXX equalsEntryValueArray
    console.time(ELAPSED_TIME_XXX); //XXX show elasped time
    console.log('TEST(3):equalsEntryValueArray(): ' + ++testCnt);
    console.log(equalsEntryValueArray([true], []));
    console.log(equalsEntryValueArray([true], [0]));
    console.log(equalsEntryValueArray([true], [false]));
    console.log(equalsEntryValueArray([true], [true]));
    console.log(equalsEntryValueArray([add3], [mul10]));
    console.log(equalsEntryValueArray([add3], [add3]));
    console.log(equalsEntryValueArray([0], [1]));
    console.log(equalsEntryValueArray([1], [1]));
    console.log(equalsEntryValueArray(['abc'], ['ABC']));
    console.log(equalsEntryValueArray(['abc'], ['abc']));
    console.log(equalsEntryValueArray([[true]], [[]]));
    console.log(equalsEntryValueArray([[true]], [[false]]));
    console.log(equalsEntryValueArray([[true]], [[true]]));
    console.log(equalsEntryValueArray([{ 'bool': true }], [{}]));
    console.log(equalsEntryValueArray([{ 'bool': true }], [{ 'Function': add3 }]));
    console.log(equalsEntryValueArray([{ 'bool': true }], [{ 'bool': true }]));
    console.timeEnd(ELAPSED_TIME_XXX); //XXX show elasped time
    //XXX equalsEntryValue
    console.time(ELAPSED_TIME_XXX); //XXX show elasped time
    console.log('TEST(3):equalsEntryValue(): ' + ++testCnt);
    console.log((0, exports.equalsEntryValue)({ 'bool': true }, {}));
    console.log((0, exports.equalsEntryValue)({ 'bool': true }, { 'Function': add3 }));
    console.log((0, exports.equalsEntryValue)({ 'bool': true }, { 'bool': true }));
    console.log((0, exports.equalsEntryValue)([{ 'bool': true }], [{ 'bool': true }]));
    console.log((0, exports.equalsEntryValue)(true, true));
    console.log((0, exports.equalsEntryValue)([true], [true]));
    console.log((0, exports.equalsEntryValue)(singleWidthToWideWidthChar, singleWidthToWideWidthChar));
    console.log((0, exports.equalsEntryValue)(singleWidthToWideWidthChar, sampleMap2));
    console.log((0, exports.equalsEntryValue)(sampleMap2, sampleMap2));
    console.timeEnd(ELAPSED_TIME_XXX); //XXX show elasped time
    //XXX isEmpty
    console.time(ELAPSED_TIME_XXX); //XXX show elasped time
    console.log('TEST(3):isEmpty(): ' + ++testCnt);
    console.log((0, exports.isEmpty)({}));
    console.log((0, exports.isEmpty)((0, exports.create)()));
    console.log((0, exports.isEmpty)(sampleMap));
    console.log((0, exports.isEmpty)(sampleMap2));
    console.log((0, exports.isEmpty)(singleWidthToWideWidthChar));
    console.log((0, exports.isEmpty)(wideWidthToSingleWidthChar));
    console.timeEnd(ELAPSED_TIME_XXX); //XXX show elasped time
    console.timeEnd(ELAPSED_TIME_TEST4); //XXX show elasped time
    //XXX cleanUp
    console.time(ELAPSED_TIME_TEST5); //XXX show elasped time
    console.log('TEST(4):cleanUp(): ' + ++testCnt);
    console.log((0, exports.cleanUp)(sampleMap, false));
    console.log((0, exports.cleanUp)(sampleMap2, false));
    console.log((0, exports.cleanUp)(singleWidthToWideWidthChar, false));
    console.log((0, exports.cleanUp)(wideWidthToSingleWidthChar, false));
    console.timeEnd(ELAPSED_TIME_TEST5); //XXX show elasped time
    console.log('...TEST(END) ' + ++testCnt);
    console.timeEnd(ELASPED_TIME_ALL); //XXX show elasped time
    console.log('XXX:perform=[' + (1 + perform) + '/' + PERFORM_ITERATE_CNT + ']');
} //XXX 何回か繰り返して性能を評価する。
//【End of tests】
